import datetime
import time


def generate_excerpt(body):
	excerpt_parts = []
	for line in body.splitlines():
		if line == '':
			break
		excerpt_parts.append(line)
	return ' '.join(excerpt_parts)


def now_datetime_str(utc=False, include_timezone=True):
	if utc:
		now = datetime.datetime.utcnow()
	else:
		now = datetime.datetime.now()

	ret = now.strftime('%Y-%m-%d %H:%M:%S')

	if include_timezone:
		if utc:
			now = time.gmtime()
		else:
			now = time.localtime()
		ret += ' ' + time.strftime('%z', now)

	return ret


def parse_pubdate(pubdate):
	formats = ('%Y-%m-%d %H:%M:%S %z', '%Y-%m-%d %H:%M:%S', '%Y-%m-%d')
	for dateformat in formats:
		try:
			return datetime.datetime.strptime(pubdate, dateformat)
		except ValueError:
			pass
	return None



import argparse
import logging
import os
import os.path

import russell
import russell.config
import russell.scaffold

LOG = logging.getLogger(__name__)


def get_args(args=None):
	parser = argparse.ArgumentParser('russell')
	parser.add_argument('-c', '--config',
		help='path to config file. defaults to $PWD/rusell.yml')
	parser.add_argument('-l', '--log-level')
	parser.add_argument('--log-file')
	subparsers = parser.add_subparsers(dest='command')

	setup_parser = subparsers.add_parser('setup')
	setup_parser.add_argument('path')
	setup_parser.add_argument('-f', '--files',
		help='specify which files to scaffold')
	setup_parser.add_argument('-o', '--overwrite', action='store_true',
		help='overwrite existing files')

	new_page_parser = subparsers.add_parser('new-page')
	new_page_parser.add_argument('title',
		help='title of the new page. wrap in quotes!')

	new_post_parser = subparsers.add_parser('new-post')
	new_post_parser.add_argument('title',
		help='title of the new post. wrap in quotes!')
	new_post_parser.add_argument('--pubdate', help='date/time of the post')
	new_post_parser.add_argument('--tags', help='comma-separated tags')

	generate_parser = subparsers.add_parser('generate')
	generate_parser.add_argument('-d', '--dist-path')
	generate_parser.add_argument('-r', '--root-url')

	return parser.parse_args(args)


def get_config(args):
	if not args.config:
		args.config = os.path.join(os.getcwd(), 'russell.yml')

	if os.path.isfile(args.config):
		return russell.config.load_config_file(args.config)

	return {}


def setup_logging(args, config):
	log_level = None
	if args.log_level:
		log_level = getattr(logging, args.log_level.upper())
	elif config.get('log_level'):
		log_level = getattr(logging, config['log_level'].upper())

	log_file = None
	if args.log_file:
		log_file = args.log_file
	elif config.get('log_file'):
		log_file = config['log_file']

	russell.setup_logging(log_file=log_file, log_level=log_level)


def main(args=None):
	args = get_args(args)
	config = get_config(args)
	setup_logging(args, config)

	if args.command == 'setup':
		if args.files:
			args.files = [file.strip() for file in args.files.split(',')]
		russell.scaffold.scaffold(args.path, args.files, args.overwrite)
		print('Done!')
		return

	if not os.path.isfile(args.config):
		raise FileNotFoundError('Config file not found - looked in ' + args.config)

	app = russell.Application(config)

	if args.command == 'new-page':
		app.create_new_page(args.title)
	elif args.command == 'new-post':
		if args.tags:
			args.tags = [tag.strip() for tag in args.tags.split(',')]
		app.create_new_post(args.title, args.pubdate, args.tags)
	elif args.command == 'generate':
		app.generate_site(args.dist_path, args.root_url)
	else:
		raise RuntimeError('This code should not be reachable!')

	print('Done!')


if __name__ == '__main__':
	main()



import logging
import os
import os.path
import sys
import yaml

import russell.parser
import russell.jinja
import russell.generator
import russell.rss
import russell.scaffold

LOG = logging.getLogger(__name__)


def setup_logging(log_file=None, log_level=None):
	if log_level is None:
		log_level = logging.WARNING
	root = logging.getLogger()
	root.setLevel(log_level)

	if not log_file or log_file == 'stdout':
		handler = logging.StreamHandler(sys.stdout)
		log_file = 'STDOUT'
	elif log_file == 'stderr':
		handler = logging.StreamHandler(sys.stderr)
		log_file = 'STDERR'
	elif log_file:
		handler = logging.StreamHandler(log_file)

	# define the logging format
	formatter = logging.Formatter('[%(asctime)s] %(levelname)s - %(name)s - %(message)s')
	handler.setFormatter(formatter)

	# add the logging handler for all loggers
	root.addHandler(handler)

	LOG.info('set up logging to %s with level %s', log_file, log_level)


class Application:
	def __init__(self, config):
		self.config = config

	def generate_site(self, dist_path=None, root_url=None):
		if dist_path is None:
			dist_path = self.config['dist_path']
		if root_url is None:
			root_url = self.config.get('root_url', '')

		pages = russell.parser.read_pages(self.config['pages_path'], root_url)
		posts = russell.parser.read_posts(self.config['posts_path'], root_url)
		tags = russell.parser.get_tags(posts)
		jinja_env = russell.jinja.make_env(self.config['templates_path'], root_url)
		russell.generator.generate(dist_path, pages, posts, tags, jinja_env)

		rss_path = os.path.join(dist_path, 'rss.xml')
		rss_title = self.config.get('title', root_url)
		russell.rss.write_feed(rss_path, posts, rss_title, root_url)

	def create_new_page(self, title):
		russell.scaffold.create_new_page(self.config['posts_path'], title)

	def create_new_post(self, title, pubdate=None, tags=None):
		russell.scaffold.create_new_post(self.config['posts_path'], title,
			pubdate=pubdate, tags=tags)



import os
import os.path
import logging

import russell.content
import russell.markdown

LOG = logging.getLogger(__name__)


def _list_files(root_dir):
	results = set()

	for root, _, files in os.walk(root_dir):
		for file in files:
			results.add(os.path.join(root, file))

	return results


def read_pages(path, root_url):
	kwargs = {'root_url': root_url}
	pages = []
	LOG.debug('looking for page files in %s', path)
	for file in _list_files(path):
		page = russell.markdown.from_file(file, russell.content.Page, kwargs)
		pages.append(page)
	return pages


def read_posts(path, root_url):
	kwargs = {'root_url': root_url}
	posts = []
	LOG.debug('looking for post files in %s', path)
	for file in _list_files(path):
		post = russell.markdown.from_file(file, russell.content.Post, kwargs)
		posts.append(post)

	posts.sort(key=lambda post: post.title)
	posts.sort(key=lambda post: post.pubdate, reverse=True)

	return posts


def get_tags(posts):
	tags = set()
	for post in posts:
		for tag in post.tags:
			tags.add(tag)
	LOG.debug('found %d unique tags', len(tags))
	return tags



import os
import os.path
import logging

LOG = logging.getLogger(__name__)


def generate(path, pages, posts, tags, jinja_env):
	if not os.path.isdir(path):
		os.mkdir(path)

	generate_pages(path, pages, jinja_env)
	generate_posts(os.path.join(path, 'posts'), posts, jinja_env)
	generate_tags(os.path.join(path, 'tags'), tags, posts, jinja_env)
	generate_home(os.path.join(path, 'index.html'), posts, jinja_env)
	generate_archive(os.path.join(path, 'archive.html'), posts, jinja_env)


def generate_pages(path, pages, jinja_env):
	template = jinja_env.select_template(['page.html.jinja', 'post.html.jinja'])

	for page in pages:
		LOG.info('generating HTML for page "%s"', page.title)
		html = template.render(page=page)
		page_path = os.path.join(path, page.slug + '.html')
		with open(page_path, 'w') as file:
			LOG.info('writing page "%s" to "%s"', page.title, page_path)
			file.write(html)


def generate_posts(path, posts, jinja_env):
	if not os.path.isdir(path):
		os.mkdir(path)

	template = jinja_env.select_template(['post.html.jinja', 'page.html.jinja'])

	for post in posts:
		LOG.info('generating HTML for post "%s"', post.title)
		html = template.render(post=post)
		post_path = os.path.join(path, post.slug + '.html')
		with open(post_path, 'w') as file:
			LOG.info('writing post "%s" to "%s"', post.title, post_path)
			file.write(html)


def generate_tags(path, tags, posts, jinja_env):
	if not os.path.isdir(path):
		os.mkdir(path)

	for tag in tags:
		tag_posts = [post for post in posts if tag in post.tags]
		if tag_posts:
			tag_path = os.path.join(path, tag.slug + '.html')
			generate_archive(tag_path, tag_posts, jinja_env)


def generate_home(path, posts, jinja_env):
	LOG.info('generating HTML for home page "%s"', path)
	template = jinja_env.select_template(['home.html.jinja', 'archive.html.jinja'])
	html = template.render(posts=posts)
	with open(path, 'w') as file:
		LOG.info('writing home page "%s"', path)
		file.write(html)


def generate_archive(path, posts, jinja_env):
	LOG.info('generating HTML for archive "%s"', path)
	template = jinja_env.select_template(['archive.html.jinja', 'home.html.jinja'])
	html = template.render(posts=posts)
	with open(path, 'w') as file:
		LOG.info('writing archive "%s"', path)
		file.write(html)



import datetime
import logging
import PyRSS2Gen

LOG = logging.getLogger(__name__)


def _get_rss_item(post):
	return PyRSS2Gen.RSSItem(
		title=post.title,
		description=post.excerpt,
		link=post.url,
		pubDate=post.pubdate,
	)


def write_feed(path, posts, title, root_url):
	LOG.info('generating RSS feed')

	rss = PyRSS2Gen.RSS2(
		title=title,
		description='',
		link=root_url,
		lastBuildDate=datetime.datetime.now(),
		items=[_get_rss_item(post) for post in posts],
	)

	with open(path, 'w') as file:
		LOG.info('writing RSS feed to "%s"', path)
		rss.write_xml(file)



import os
import os.path
import yaml


class ConfigFileNotFoundError(FileNotFoundError):
	pass


def normalize_config_dict(data):
	if 'root_path' not in data:
		data['root_path'] = os.getcwd()

	def set_dir_config(name):
		if name + '_dir' not in data:
			data[name + '_dir'] = name
		data[name + '_path'] = os.path.join(data['root_path'], data[name + '_dir'])

	set_dir_config('posts')
	set_dir_config('pages')
	set_dir_config('templates')
	set_dir_config('dist')

	return data


def load_config_file(path):
	with open(path) as file:
		return normalize_config_dict(yaml.load(file))



import logging
import jinja2

LOG = logging.getLogger(__name__)


def make_env(template_path, root_url):
	LOG.debug('setting up jinja env with root dir "%s"', template_path)
	loader = jinja2.FileSystemLoader(template_path)
	jinja_env = jinja2.Environment(loader=loader)
	jinja_env.globals['root_url'] = root_url
	return jinja_env



import logging
import os
import os.path
import slugify

import russell.util

LOG = logging.getLogger(__name__)


DEFAULT_CONFIG = '''
root_path: {pwd}
root_url: //localhost:8080
title: My blog

# these are relative to root_path. you can remove them
# or comment them out to leave them as their defaults.
#posts_dir: posts
#pages_dir: pages
#dist_dir: dist
#templates_dir: templates
'''

LAYOUT_TEMPLATE = '''
<!doctype html>
<html lang="en">
<head>
	<meta charset="UTF-8">
	<title>{% block title %}{% endblock %} - Russell generated</title>
	<link rel="alternate" type="application/rss+xml" href="{{ root_url }}/rss.xml">
	<link rel="stylesheet" href="{{ root_url }}/static/style.css">
</head>
<body>
	<header class="site-header">
		<h1 class="site-title">
			<a href="{{ root_url }}">My website</a>
		</h1>
	</header>
	<main class="site-main">
		{% block content %}{% endblock %}
	</main>
</body>
</html>
'''

HOME_TEMPLATE = '''
{% extends 'layout.html.jinja' %}
{% block title %}Archives{% endblock %}
{% block content %}
{% for post in posts %}
	<article class="post">
		<header class="post-header">
			<span class="post-pubdate">{{ post.pubdate.strftime('%B %d, %Y') }}</span>
			<h2 class="post-title">
				<a href="{{ post.url }}">{{ post.title }}</a>
			</h2>
			<span class="post-tags">{{ ', '.join(post.tag_links) }}</span>
		</header>
		<div class="post-body">{{ post.excerpt }}</div>
	</article>
{% endfor %}
{% endblock %}
'''

PAGE_TEMPLATE = '''
{% extends 'layout.html.jinja' %}
{% block title %}{{ page.title }}{% endblock %}
{% block content %}
<article class="page">
	<header class="page-header">
		<h1 class="page-title">{{ page.title }}</h1>
	</header>
	<div class="page-body">{{ page.body }}</div>
</article>
{% endblock %}
'''

POST_TEMPLATE = '''
{% extends 'layout.html.jinja' %}
{% block title %}{{ post.title }}{% endblock %}
{% block content %}
<article class="post">
	<header class="post-header">
		<h2 class="post-title">{{ post.title }}</h2>
		<span class="post-pubdate">{{ post.pubdate.strftime('%B %d, %Y') }}</span>
	</header>
	<div class="post-body">{{ post.body }}</div>
	<span class="post-tags">{{ ', '.join(post.tag_links) }}</span>
</article>
{% endblock %}
'''

SAMPLE_PAGE = '''
# About me

Write something about you here!
'''

SAMPLE_POST = '''
# Hello world!
pubdate: {pubdate}
tags: tag1, tag2

This is my first Russell blogpost!
'''


class ScaffoldingException(Exception):
	pass


def _create_dir(path):
	if os.path.isdir(path):
		LOG.info('directory already exists, not doing anything: %s', path)
	else:
		LOG.info('creating directory: %s', path)
		os.mkdir(path)


def get_page_slug_and_contents(title, body):
	contents = '# {}\n\n{}'.format(title, body)

	return slugify.slugify(title), contents


def create_new_page(path, title):
	slug, contents = get_page_slug_and_contents(title, 'Write something here.')
	path = os.path.join(path, slug + '.md')
	if os.path.isfile(path):
		raise ScaffoldingException('File already exists: {}'.format(path))

	LOG.info('writing page to "%s"', path)
	with open(path, 'w') as file:
		file.write(contents + '\n')


def get_post_slug_and_contents(title, body, pubdate=None, tags=None):
	contents = '# {}\n'.format(title)

	if pubdate is None:
		pubdate = russell.util.now_datetime_str()
	contents += 'pubdate: {}\n'.format(pubdate)

	if tags:
		contents += 'tags: {}\n'.format(', '.join(tags))

	contents += '\n{}'.format(body)

	return slugify.slugify(title), contents


def create_new_post(path, title, **kwargs):
	slug, contents = get_post_slug_and_contents(title, 'Write something here.', **kwargs)
	path = os.path.join(path, slug + '.md')
	if os.path.isfile(path):
		raise ScaffoldingException('File already exists: {}'.format(path))

	LOG.info('writing post to "%s"', path)
	with open(path, 'w') as file:
		file.write(contents + '\n')


def scaffold(path, files=None, overwrite=False):
	_create_dir(path)
	scaffolder = Scaffolder(path, overwrite)
	if files is None or 'config' in files:
		scaffolder.create_config()
	if files is None or 'templates' in files:
		scaffolder.create_templates()
	if files is None or 'posts' in files:
		scaffolder.create_posts()
	if files is None or 'pages' in files:
		scaffolder.create_pages()


class Scaffolder:
	def __init__(self, root_path, overwrite=False):
		self.root_path = root_path
		self.overwrite = overwrite

	def _create_file(self, path, contents):
		if os.path.isfile(path) and not self.overwrite:
			LOG.info('file already exists, not doing anything: %s', path)
		else:
			LOG.info('writing file: %s', path)
			with open(path, 'w') as file:
				file.write(contents.lstrip())

	def create_config(self):
		path = os.path.join(self.root_path, 'russell.yml')
		self._create_file(path, DEFAULT_CONFIG.format(pwd=os.getcwd()))

	def create_templates(self):
		path = os.path.join(self.root_path, 'templates')
		_create_dir(path)

		home_path = os.path.join(path, 'home.html.jinja')
		self._create_file(home_path, HOME_TEMPLATE)

		layout_path = os.path.join(path, 'layout.html.jinja')
		self._create_file(layout_path, LAYOUT_TEMPLATE)

		page_path = os.path.join(path, 'page.html.jinja')
		self._create_file(page_path, PAGE_TEMPLATE)

		post_path = os.path.join(path, 'post.html.jinja')
		self._create_file(post_path, POST_TEMPLATE)

	def create_pages(self):
		path = os.path.join(self.root_path, 'pages')
		_create_dir(path)
		self._create_file(os.path.join(path, 'about.md'), SAMPLE_PAGE)

	def create_posts(self):
		path = os.path.join(self.root_path, 'posts')
		_create_dir(path)
		contents = SAMPLE_POST.format(pubdate=russell.util.now_datetime_str())
		self._create_file(os.path.join(path, 'hello-world.md'), contents)



import slugify

import russell.util


class Page:
	def __init__(self, title, body, slug=None, root_url=None):
		self.title = title
		self.body = body
		self.slug = slug or slugify.slugify(title)
		self.root_url = root_url

	@property
	def url(self):
		return self.root_url + '/' + self.slug


class Post(Page):
	# pylint: disable=too-many-arguments
	def __init__(self, title, body, pubdate=None, slug=None, excerpt=None,
			tags=None, root_url=None):
		super().__init__(title, body, slug, root_url)
		self.excerpt = excerpt or russell.util.generate_excerpt(body)
		self.pubdate = pubdate
		self.tags = tags or []
	# pylint: enable=too-many-arguments

	@property
	def url(self):
		return self.root_url + '/posts/' + self.slug

	@property
	def tag_links(self):
		return ['<a href="' + tag.url + '">' + tag.title + '</a>' for tag in self.tags]


class Tag:
	def __init__(self, title, slug=None, root_url=None):
		self.title = title
		self.slug = slug or slugify.slugify(title)
		self.root_url = root_url

	@property
	def url(self):
		return self.root_url + '/tags/' + self.slug



import datetime
import os.path
import logging
import markdown

import russell.content
import russell.util

LOG = logging.getLogger(__name__)


def from_file_contents(contents, cls, kwargs=None):
	if kwargs is None:
		kwargs = {}

	lines = contents.splitlines()
	title = None

	line = lines.pop(0)
	while line != '':
		if not title and line.startswith('#'):
			title = line.replace('#', '').strip()
		elif line.startswith('title:'):
			title = line[6:].strip()

		elif line.startswith('slug:'):
			kwargs['slug'] = line[5:].strip()

		elif line.startswith('pubdate:') and cls is russell.content.Post:
			pubdate_str = line[8:].strip()
			pubdate = russell.util.parse_pubdate(pubdate_str)
			if pubdate:
				kwargs['pubdate'] = pubdate
			else:
				LOG.warning('Could not parse datetime "%s"', pubdate_str)

		elif line.startswith('tags:') and cls is russell.content.Post:
			kwargs['tags'] = [
				russell.content.Tag(tag.strip(), root_url=kwargs.get('root_url', ''))
				for tag in line[5:].strip().split(',')
			]

		line = lines.pop(0)

	# the only lines left should be the actual contents
	body = '\n'.join(lines).strip()
	if cls is russell.content.Post:
		kwargs['excerpt'] = markdown.markdown(
			russell.util.generate_excerpt(body)
		)
	body = markdown.markdown(body)

	return cls(title=title, body=body, **kwargs)


def from_file(path, cls, kwargs=None):
	if kwargs is None:
		kwargs = {}

	LOG.debug('creating %s from "%s"', cls, path)

	# the filename will be the default slug - can be overridden later
	kwargs['slug'] = os.path.splitext(os.path.basename(path))[0]

	with open(path, 'r') as file:
		post = from_file_contents(file.read(), cls, kwargs)

	# if a pubdate wasn't found, use the file's last modified time
	if cls is russell.content.Post and not kwargs.get('pubdate'):
		timestamp = min(os.path.getctime(path), os.path.getmtime(path))
		kwargs['pubdate'] = datetime.datetime.fromtimestamp(timestamp)

	return post
